@@ -5,7 +5,7 @@
  * license that can be found in the BSD_LICENSE file.
  */
 
-/* sg_pt_linux version 1.20 20131014 */
+/* sg_pt_linux version 1.22 20140606 */
 
 
 #include <stdio.h>
@@ -287,15 +287,14 @@ set_scsi_pt_flags(struct sg_pt_base * vp, int flags)
 {
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    /* default action of SG (v3) is QUEUE_AT_HEAD */
+    /* default action of sg driver [sg v3 interface] is QUEUE_AT_HEAD */
     /* default action of block layer SG_IO ioctl is QUEUE_AT_TAIL */
-    if (SCSI_PT_FLAGS_QUEUE_AT_TAIL & flags) {
-        ptp->io_hdr.flags |= SG_FLAG_Q_AT_TAIL;
-        ptp->io_hdr.flags &= ~SG_FLAG_Q_AT_HEAD;
-    }
-    if (SCSI_PT_FLAGS_QUEUE_AT_HEAD & flags) {
+    if (SCSI_PT_FLAGS_QUEUE_AT_HEAD & flags) {  /* favour AT_HEAD */
         ptp->io_hdr.flags |= SG_FLAG_Q_AT_HEAD;
         ptp->io_hdr.flags &= ~SG_FLAG_Q_AT_TAIL;
+    } else if (SCSI_PT_FLAGS_QUEUE_AT_TAIL & flags) {
+        ptp->io_hdr.flags |= SG_FLAG_Q_AT_TAIL;
+        ptp->io_hdr.flags &= ~SG_FLAG_Q_AT_HEAD;
     }
 }
 
@@ -754,17 +753,32 @@ set_scsi_pt_task_attr(struct sg_pt_base * vp, int attribute, int priority)
 #ifndef BSG_FLAG_Q_AT_TAIL
 #define BSG_FLAG_Q_AT_TAIL 0x10
 #endif
+#ifndef BSG_FLAG_Q_AT_HEAD
+#define BSG_FLAG_Q_AT_HEAD 0x20
+#endif
+
+/* Need this later if translated to v3 interface */
+#ifndef SG_FLAG_Q_AT_TAIL
+#define SG_FLAG_Q_AT_TAIL 0x10
+#endif
+#ifndef SG_FLAG_Q_AT_HEAD
+#define SG_FLAG_Q_AT_HEAD 0x20
+#endif
 
 void
 set_scsi_pt_flags(struct sg_pt_base * vp, int flags)
 {
     struct sg_pt_linux_scsi * ptp = &vp->impl;
 
-    /* default action of bsg (sg v4) is QUEUE_AT_HEAD */
-    if (SCSI_PT_FLAGS_QUEUE_AT_TAIL & flags)
-        ptp->io_hdr.flags |= BSG_FLAG_Q_AT_TAIL;
-    if (SCSI_PT_FLAGS_QUEUE_AT_HEAD & flags)
+    /* default action of bsg driver (sg v4) is QUEUE_AT_HEAD */
+    /* default action of block layer SG_IO ioctl is QUEUE_AT_TAIL */
+    if (SCSI_PT_FLAGS_QUEUE_AT_HEAD & flags) {  /* favour AT_HEAD */
+        ptp->io_hdr.flags |= BSG_FLAG_Q_AT_HEAD;
         ptp->io_hdr.flags &= ~BSG_FLAG_Q_AT_TAIL;
+    } else if (SCSI_PT_FLAGS_QUEUE_AT_TAIL & flags) {
+        ptp->io_hdr.flags |= BSG_FLAG_Q_AT_TAIL;
+        ptp->io_hdr.flags &= ~BSG_FLAG_Q_AT_HEAD;
+    }
 }
 
 /* N.B. Returns din_resid and ignores dout_resid */
@@ -931,6 +945,10 @@ do_scsi_pt_v3(struct sg_pt_linux_scsi * ptp, int fd, int time_secs,
         v3_hdr.mx_sb_len = (unsigned char)ptp->io_hdr.max_response_len;
     }
     v3_hdr.pack_id = (int)ptp->io_hdr.spare_in;
+    if (BSG_FLAG_Q_AT_HEAD & ptp->io_hdr.flags)
+        v3_hdr.flags |= SG_FLAG_Q_AT_HEAD;      /* favour AT_HEAD */
+    else if (BSG_FLAG_Q_AT_TAIL & ptp->io_hdr.flags)
+        v3_hdr.flags |= SG_FLAG_Q_AT_TAIL;
 
     if (NULL == v3_hdr.cmdp) {
         if (verbose)
