@@ -84,6 +84,9 @@ extern "C" {
 #define MAX_XC_BPT_POW2 32768  /* BPT maximum that is power of 2 */
 #define DEF_SCSI_CDBSZ 10
 #define MAX_SCSI_CDBSZ 32
+#define DDPT_MAX_JF_DEPTH 4
+#define DDPT_MAX_JF_LINES 1000
+#define DDPT_MAX_JF_ARGS_PER_LINE 16
 
 #define VPD_DEVICE_ID 0x83
 #define VPD_3PARTY_COPY 0x8f
@@ -119,6 +122,7 @@ extern "C" {
 #define FT_FIFO 64              /* fifo (named or unnamed pipe (stdout)) */
 #define FT_CHAR 128             /* char dev, doesn't fit another category */
 #define FT_ERROR 256            /* couldn't "stat" file */
+#define FT_ALL_FF 512           /* iflag=ff so input will be 0xff bytes */
 
 /* ODX type requested */
 #define ODX_REQ_NONE 0          /* some other type of copy */
@@ -167,7 +171,6 @@ extern "C" {
 #define SG_LIB_CAT_PROTECTION_WITH_INFO 41
 #endif
 
-#define DDPT_CAT_RESERVATION_CONFLICT 30
 #define DDPT_CAT_PARAM_LST_LEN_ERR 50
 #define DDPT_CAT_INVALID_FLD_IN_PARAM 51
 #define DDPT_CAT_TOO_MANY_SEGS_IN_PARAM 52
@@ -178,13 +181,12 @@ extern "C" {
 #define DDPT_CAT_INSUFF_RES_CREATE_RODTOK 57
 #define DDPT_CAT_CMDS_CLEARED_BY_DEV_SVR 58
 #define DDPT_CAT_TOKOP_BASE 70        /* assume less than 20 above this */
-#define DDPT_CAT_SK_DATA_PROTECT 7    /* same as sense key code */
-#define DDPT_CAT_SK_COPY_ABORTED 10   /* same as sense key code */
 
 #define XCOPY_TO_SRC "XCOPY_TO_SRC"
 #define XCOPY_TO_DST "XCOPY_TO_DST"
 #define DEF_XCOPY_SRC0_DST1 1
 #define ODX_RTF_LEN "ODX_RTF_LEN"     /* append 8 byte ROD size to token */
+#define DDPT_DEF_BS "DDPT_DEF_BS" /* default default block size: 512 bytes */
 
 /* ODX: length field inside ROD Token constant, implies 512 byte ROD Token */
 #define ODX_ROD_TOK_LEN_FLD 504       /* 0x1f8 */
@@ -232,7 +234,9 @@ struct block_rodtok_vpd {
  * General or for disk unless otherwise marked. */
 struct flags_t {
     int append;
+    int atomic;
     int block;          /* only for pt, non blocking default */
+    int bytchk;         /* set field in WRITE AND VERIFY */
     int cat;            /* xcopy(lid1) related */
     int cdbsz;
     int coe;
@@ -243,6 +247,7 @@ struct flags_t {
     int errblk;
     int excl;
     int fdatasync;
+    int ff;             /* iflag=ff makes input all 0xff bytes */
     int flock;
     int force;
     int fsync;
@@ -271,6 +276,7 @@ struct flags_t {
     int strunc;
     int sync;
     int trunc;
+    int verify;         /* oflag with pt, turns WRITE into WRITE AND VERIFY */
     int wsame16;
     int xcopy;          /* xcopy(LID1) */
 };
@@ -354,8 +360,10 @@ struct opts_t {
     struct dev_info_t * o2dip;
     /* working variables and statistics */
     int64_t dd_count;   /* -1 for not specified, 0 for no blocks to copy */
-    int64_t in_full;
-    int64_t out_full;
+                        /* after copy/read starts, decrements to 0 */
+    int64_t dd_count_start;     /* dd_count prior to start of copy/read */
+    int64_t in_full;    /* full blocks read from IFILE so far */
+    int64_t out_full;   /* full blocks written to OFILE so far */
     int64_t out_sparse; /* used for sparse, sparing + trim */
     int64_t lowest_unrecovered;         /* on reads */
     int64_t highest_unrecovered;        /* on reads */
@@ -386,6 +394,7 @@ struct opts_t {
     int reading_fifo;
     int read1_or_transfer; /* 1 when of=/dev/null or similar */
     int ibs_hold;
+    int o_readonly;
     unsigned char * wrkBuff;
     unsigned char * wrkPos;
     unsigned char * wrkBuff2;
@@ -537,7 +546,7 @@ int do_odx(struct opts_t * op);
 
 /* defined in ddpt_cl.c */
 int cl_process(struct opts_t * op, int argc, char * argv[],
-               const char * version_str);
+               const char * version_str, int jf_depth);
 void ddpt_usage(int help);
 
 
