@@ -221,8 +221,22 @@ print_stats(const char * str, struct opts_t * op, int who)
     print_tape_summary(op, 0, str);
 #endif
 
-    if ((op->dd_count > 0) && (! op->reading_fifo))
-        pr2serr("  remaining block count=%" PRId64 "\n", op->dd_count);
+    if ((op->dd_count > 0) && (! op->reading_fifo)) {
+        pr2serr("  remaining block count=%" PRId64, op->dd_count);
+        if ((who < 2) && (op->in_full <= op->dd_count_start) &&
+            (op->dd_count_start >= 4196)) {
+            /* with ints will overflow around 8 TB for bs=1 */
+            int num = (int)((op->in_full * 100) / 4196);
+            int den = (int)(op->dd_count_start / 4196);  /* will be >= 1 */
+            int percent;
+
+            percent = num / den;
+            if ((100 == percent) && (op->in_full < op->dd_count_start))
+                --percent;      /* don't want rounding to 100 % */
+            pr2serr("   %d%% completed\n", percent);
+        } else
+            pr2serr("\n");
+    }
     if (who < 2)
         pr2serr("%s%" PRId64 "+%d records in\n", str, op->in_full,
                 op->in_partial);
@@ -303,12 +317,12 @@ find_bsg_major(int verbose)
         return;
     }
     while ((cp = fgets(b, sizeof(b), fp))) {
-        if ((1 == sscanf(b, "%s", a)) &&
+        if ((1 == sscanf(b, "%126s", a)) &&
             (0 == memcmp(a, "Character", 9)))
             break;
     }
     while (cp && (cp = fgets(b, sizeof(b), fp))) {
-        if (2 == sscanf(b, "%d %s", &n, a)) {
+        if (2 == sscanf(b, "%d %126s", &n, a)) {
             if (0 == strcmp("bsg", a)) {
                 bsg_major = n;
                 break;
@@ -1194,10 +1208,20 @@ signals_process_delay(struct opts_t * op, int delay_type)
             /* Don't show next message if using oflag=pre-alloc and we didn't
              * use FALLOC_FL_KEEP_SIZE */
             if ((0 == op->reading_fifo) && (FT_REG & op->odip->d_type_hold)
-                 && (0 == op->oflagp->prealloc))
+                 && (0 == op->oflagp->prealloc)) {
+                if (op->oflagp->strunc) {
+                    int64_t osize = (op->seek * op->obs);
+
+                    pr2serr(">> Did not perform: "
+                            "'truncate --size=%" PRId64 " %s'\n",
+                            osize, op->odip->fn);
+                    pr2serr(">> User should check and perform by hand if "
+                            "necessary\n");
+                }
                 pr2serr("To resume, invoke with same arguments plus "
                         "oflag=resume\n");
-            ; // >>>>>>>>>>>>> cleanup ();
+            }
+            // Could more cleanup or suggestions be made here?
         } else {
             pr2serr("Progress report:\n");
             print_stats("  ", op, 0);
@@ -1361,7 +1385,7 @@ decode_designation_descriptor(const unsigned char * ucp, int len_less_4,
         case 3:         /* NAA: Locally assigned */
         case 5:         /* NAA: IEEE Registered */
             if (8 != i_len) {
-                print_p("      << unexpected NAA 5 identifier length: "
+                print_p("      << unexpected NAA 3 or 5 identifier length: "
                         "0x%x>>\n", i_len);
                 dStrHexp((const char *)ip, i_len, -1);
                 break;
@@ -1605,13 +1629,13 @@ print_exit_status_msg(const char * prefix, int exit_stat, int to_stderr)
     case SG_LIB_CAT_UNIT_ATTENTION:   /* 6 */
         print_p("%sunit attention\n", b);
         break;
-    case DDPT_CAT_SK_DATA_PROTECT:   /* 7 */
+    case SG_LIB_CAT_DATA_PROTECT:   /* 7 */
         print_p("%sdata protect\n", b);
         break;
     case SG_LIB_CAT_INVALID_OP:   /* 9 */
         print_p("%sinvalid opcode\n", b);
         break;
-    case DDPT_CAT_SK_COPY_ABORTED:   /* 10 */
+    case SG_LIB_CAT_COPY_ABORTED:   /* 10 */
         print_p("%scopy aborted\n", b);
         break;
     case SG_LIB_CAT_ABORTED_COMMAND:   /* 11 */
@@ -1629,11 +1653,11 @@ print_exit_status_msg(const char * prefix, int exit_stat, int to_stderr)
     case SG_LIB_CAT_RECOVERED:   /* 21 */
         print_p("%srecovered error (possible future errors)\n", b);
         break;
-    case DDPT_CAT_RESERVATION_CONFLICT:   /* 30 */
-        print_p("%sSCSI command timeout\n", b);
+    case SG_LIB_CAT_RES_CONFLICT:   /* 24 */
+        print_p("%sSCSI status: reservation conflict\n", b);
         break;
     case SG_LIB_CAT_TIMEOUT:   /* 33 */
-        print_p("%sSCSI status: reservation conflict\n", b);
+        print_p("%sSCSI command timeout\n", b);
         break;
     case SG_LIB_CAT_PROTECTION:   /* 40 */
         print_p("%sprotection error\n", b);
@@ -1756,7 +1780,7 @@ cl_to_sgl(const char * inp, struct scat_gath_elem * sgl_arr,
         pr2serr("'--lba' cannot be read from stdin\n");
         return 1;
     } else {        /* list of numbers (default decimal) on command line */
-        k = strspn(inp, "0123456789aAbBcCdDeEfFhHxXbBdDiIkKmMgGtTpP, ");
+        k = strspn(inp, "0123456789aAbBcCdDeEfFhHxXiIkKmMgGtTpP, ");
         if (in_len != k) {
             pr2serr("cl_to_sgl: error at pos %d\n", k + 1);
             return 1;
