@@ -74,6 +74,9 @@
 #define DDPT_WRITE10_OC 0x2a
 #define DDPT_WRITE12_OC 0xaa
 #define DDPT_WRITE16_OC 0x8a
+#define DDPT_WRITE_ATOMIC16_OC 0x9c     /* sbc4r02 */
+#define DDPT_WRITE_VERIFY10_OC 0x2e
+#define DDPT_WRITE_VERIFY16_OC 0x8e
 #define DDPT_VARIABLE_LEN_OC 0x7f
 #define DDPT_READ32_SA 0x9
 #define DDPT_WRITE32_SA 0xb
@@ -89,8 +92,6 @@
 #define ASQ_TARGET_UNDERRUN 0x4
 #define ASQ_TARGET_OVERRUN 0x5
 
-#define ASC_PROTECTION_INFO_BAD 0x10
-
 #define ASC_PARAM_LST_LEN_ERR 0x1a      /* only asq=00 defined */
 
 #define ASC_INVALID_TOKOP 0x23
@@ -147,18 +148,28 @@ pt_open_if(struct opts_t * op, struct sg_simple_inquiry_resp * sirp)
         flags |= O_EXCL;
     if (fp->sync)
         flags |= O_SYNC;
-    fl = O_RDWR;
+    fl = op->o_readonly ? O_RDONLY : O_RDWR;
     if ((fd = scsi_pt_open_flags(fn, (fl | flags), op->verbose)) < 0) {
         if (-EBUSY == fd) {
             pr2serr("open %s for pass-through reports BUSY,\n"
                     "  use iflag=block to wait until ready\n", fn);
             return -1;
         }
-        fl = O_RDONLY;
-        if ((fd = scsi_pt_open_flags(fn, (fl | flags), op->verbose)) < 0) {
-            pr2serr("could not open %s for pass-through: %s\n", fn,
-                    safe_strerror(-fd));
+        if (op->o_readonly) {
+            pr2serr("could not open %s ro as pass-through: %s\n", fn,
+                        safe_strerror(-fd));
             return -1;
+        } else {
+            fl = O_RDONLY;
+            if (op->verbose)
+                pr2serr("could not open %s read-write so try read-only\n",
+                        fn);
+            if ((fd = scsi_pt_open_flags(fn, (fl | flags), op->verbose))
+                < 0) {
+                pr2serr("could not open %s [rw or ro] as pass-through: %s\n",
+                        fn, safe_strerror(-fd));
+                return -1;
+            }
         }
     }
     if (sg_simple_inquiry(fd, &sir, 0, verb)) {
@@ -310,33 +321,64 @@ pt_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
                        DDPT_READ16_OC, DDPT_READ32_SA};
     int wr_opcode[] = {DDPT_WRITE6_OC, DDPT_WRITE10_OC, DDPT_WRITE12_OC,
                        DDPT_WRITE16_OC, DDPT_WRITE32_SA};
-    int sz_ind, options_byte, rw_sa;
+    int opcode_sa, options_byte, rw_sa;
+    int * opc_arr;
 
-    memset(cdbp, 0, cdb_sz);
-    options_byte = 0;
     if (cdb_sz < 6) {
         pr2serr("cdb_sz too small\n");
         return 1;
     }
+    memset(cdbp, 0, cdb_sz);
+    opcode_sa = 0;
+    options_byte = 0;
+    opc_arr = NULL;
+
+    if (write_true) {
+        if (fp->atomic) {
+            if (16 == cdb_sz)
+                opcode_sa = DDPT_WRITE_ATOMIC16_OC;
+            else {
+                pr2serr("atomic flag only for WRITE_ATOMIC(16)\n");
+                return 1;
+            }
+        } else if (fp->verify) {
+            if (10 == cdb_sz)
+                opcode_sa = DDPT_WRITE_VERIFY10_OC;
+            else if (16 == cdb_sz)
+                opcode_sa = DDPT_WRITE_VERIFY16_OC;
+            else {
+                pr2serr("verify flag for WRITE AND VERIFY (10 or 16) only\n");
+                return 1;
+            }
+        } else
+            opc_arr = wr_opcode;
+    } else {    /* a READ */
+        opc_arr = rd_opcode;
+        if (fp->rarc)
+            options_byte |= 0x4;
+    }
+
     if (cdb_sz > 6) {
         if (fp->dpo)
             options_byte |= 0x10;
-        if (fp->fua)
-            options_byte |= 0x8;
-        if (fp->fua_nv)
-            options_byte |= 0x2;
+        if (fp->verify && write_true) {
+            if (fp->bytchk)
+                options_byte |= ((fp->bytchk & 0x3) << 1);
+        } else {
+            if (fp->fua)
+                options_byte |= 0x8;
+            if (fp->fua_nv)
+                options_byte |= 0x2;
+        }
         if (protect)
             options_byte |= ((protect & 0x7) << 5);
     }
-    if ((! write_true) && fp->rarc)
-        options_byte |= 0x4;
 
     switch (cdb_sz) {
     case 6:
-        sz_ind = 0;
-        cdbp[0] = (unsigned char)(write_true ? wr_opcode[sz_ind] :
-                                               rd_opcode[sz_ind]);
-        /* Overwrite fua, fua_nv and dpo settings, n/a for 6 byte variants */
+        if (0 == opcode_sa)
+            opcode_sa = opc_arr[0];
+        cdbp[0] = (unsigned char)opcode_sa;
         cdbp[1] = (unsigned char)((start_block >> 16) & 0x1f);
         cdbp[2] = (unsigned char)((start_block >> 8) & 0xff);
         cdbp[3] = (unsigned char)(start_block & 0xff);
@@ -357,9 +399,9 @@ pt_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
         }
         break;
     case 10:
-        sz_ind = 1;
-        cdbp[0] = (unsigned char)(write_true ? wr_opcode[sz_ind] :
-                                               rd_opcode[sz_ind]);
+        if (0 == opcode_sa)
+            opcode_sa = opc_arr[1];
+        cdbp[0] = (unsigned char)opcode_sa;
         cdbp[1] = (unsigned char)options_byte;
         cdbp[2] = (unsigned char)((start_block >> 24) & 0xff);
         cdbp[3] = (unsigned char)((start_block >> 16) & 0xff);
@@ -374,9 +416,9 @@ pt_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
         }
         break;
     case 12:
-        sz_ind = 2;
-        cdbp[0] = (unsigned char)(write_true ? wr_opcode[sz_ind] :
-                                               rd_opcode[sz_ind]);
+        if (0 == opcode_sa)
+            opcode_sa = opc_arr[2];
+        cdbp[0] = (unsigned char)opcode_sa;
         cdbp[1] = (unsigned char)options_byte;
         cdbp[2] = (unsigned char)((start_block >> 24) & 0xff);
         cdbp[3] = (unsigned char)((start_block >> 16) & 0xff);
@@ -388,9 +430,9 @@ pt_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
         cdbp[9] = (unsigned char)(blocks & 0xff);
         break;
     case 16:
-        sz_ind = 3;
-        cdbp[0] = (unsigned char)(write_true ? wr_opcode[sz_ind] :
-                                               rd_opcode[sz_ind]);
+        if (0 == opcode_sa)
+            opcode_sa = opc_arr[3];
+        cdbp[0] = (unsigned char)opcode_sa;
         cdbp[1] = (unsigned char)options_byte;
         cdbp[2] = (unsigned char)((start_block >> 56) & 0xff);
         cdbp[3] = (unsigned char)((start_block >> 48) & 0xff);
@@ -406,10 +448,11 @@ pt_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
         cdbp[13] = (unsigned char)(blocks & 0xff);
         break;
     case 32:
-        sz_ind = 4;
+        if (0 == opcode_sa)
+            opcode_sa = opc_arr[4];
         cdbp[0] = (unsigned char)DDPT_VARIABLE_LEN_OC;
         cdbp[7] = (unsigned char)0x18;  /* additional length=>32 byte cdb */
-        rw_sa = write_true ? wr_opcode[sz_ind] : rd_opcode[sz_ind];
+        rw_sa = opcode_sa;
         cdbp[8] = (unsigned char)((rw_sa >> 8) & 0xff);
         cdbp[9] = (unsigned char)(rw_sa & 0xff);
         cdbp[10] = (unsigned char)options_byte;
@@ -436,10 +479,9 @@ pt_build_scsi_cdb(unsigned char * cdbp, int cdb_sz, unsigned int blocks,
 
 /* Read using the pass-through. No retries or remedial work here.
  * 0 -> successful, SG_LIB_SYNTAX_ERROR -> unable to build cdb,
- * SG_LIB_CAT_UNIT_ATTENTION -> try again,
  * SG_LIB_CAT_MEDIUM_HARD_WITH_INFO -> 'io_addrp' written to,
  * SG_LIB_CAT_MEDIUM_HARD -> no info field,
- * SG_LIB_CAT_NOT_READY, SG_LIB_CAT_ABORTED_COMMAND,
+ * plus various other SG_LIB_CAT_* positive values,
  * -2 -> ENOMEM
  * -1 other errors */
 static int
@@ -497,21 +539,19 @@ pt_low_read(struct opts_t * op, int in0_out1, unsigned char * buff,
         switch (sense_cat) {
         case SG_LIB_CAT_NOT_READY:
         case SG_LIB_CAT_INVALID_OP:
+        case SG_LIB_CAT_RES_CONFLICT:
+        case SG_LIB_CAT_DATA_PROTECT:
+        case SG_LIB_CAT_ABORTED_COMMAND:
             ++op->unrecovered_errs;
             break;
         case SG_LIB_CAT_UNIT_ATTENTION:
             break;
-        case SG_LIB_CAT_ABORTED_COMMAND:
-            if (sg_scsi_normalize_sense(sense_b, slen, &ssh) &&
-                (ASC_PROTECTION_INFO_BAD == ssh.asc)) {
-                /* Protection problem, so no retry */
-                ++op->unrecovered_errs;
-                info_valid = sg_get_sense_info_fld(sense_b, slen, io_addrp);
-                if (info_valid)
-                    ret = SG_LIB_CAT_PROTECTION_WITH_INFO;
-                else
-                    ret = SG_LIB_CAT_PROTECTION;
-            }
+        case SG_LIB_CAT_PROTECTION:
+            /* no retry, might have INFO field */
+            ++op->unrecovered_errs;
+            info_valid = sg_get_sense_info_fld(sense_b, slen, io_addrp);
+            if (info_valid)
+                ret = SG_LIB_CAT_PROTECTION_WITH_INFO;
             break;
         case SG_LIB_CAT_RECOVERED:
             ++op->recovered_errs;
@@ -594,17 +634,20 @@ pt_read(struct opts_t * op, int in0_out1, unsigned char * buff, int blocks,
     int retries_tmp;
     int ret = 0;
     int may_coe = 0;
+    const char * iop;
 
     if (in0_out1) {
         from_block = op->seek;
         bs = op->obs_pi;
         pi_len = op->obs_pi - op->obs;
         fp = op->oflagp;
+        iop = "ofile";
     } else {
         from_block = op->skip;
         bs = op->ibs_pi;
         pi_len = op->ibs_pi - op->ibs;
         fp = op->iflagp;
+        iop = "ifile";
     }
     retries_tmp = fp->retries;
     for (xferred = 0, blks = blocks, lba = from_block, bp = buff;
@@ -623,21 +666,25 @@ pt_read(struct opts_t * op, int in0_out1, unsigned char * buff, int blocks,
         case -2:        /* ENOMEM */
             return res;
         case SG_LIB_CAT_NOT_READY:
-            pr2serr("Device (r) not ready\n");
+            pr2serr("%s: Device not ready [%s]\n", __func__, iop);
             return res;
         case SG_LIB_CAT_ABORTED_COMMAND:
             if (--op->max_aborted > 0)
-                pr2serr("Aborted command, continuing (r)\n");
+                pr2serr("%s: Aborted command, continuing [%s]\n", __func__,
+                        iop);
             else {
-                pr2serr("Aborted command, too many (r)\n");
+                pr2serr("%s: Aborted command, too many [%s]\n", __func__,
+                        iop);
                 return res;
             }
             break;
         case SG_LIB_CAT_UNIT_ATTENTION:
             if (--op->max_uas > 0)
-                pr2serr("Unit attention, continuing (r)\n");
+                pr2serr("%s: Unit attention, continuing [%s]\n", __func__,
+                        iop);
             else {
-                pr2serr("Unit attention, too many (r)\n");
+                pr2serr("%s: Unit attention, too many [%s]\n", __func__,
+                        iop);
                 return res;
             }
             break;
@@ -670,6 +717,9 @@ pt_read(struct opts_t * op, int in0_out1, unsigned char * buff, int blocks,
             if (0 == retries_tmp)
                 errblk_put_range(lba, blks, op);
             /* fall through */
+        case SG_LIB_CAT_RES_CONFLICT:
+        case SG_LIB_CAT_DATA_PROTECT:
+        case SG_LIB_CAT_PROTECTION:
         default:
             if (retries_tmp > 0) {
                 pr2serr(">>> retrying pt read: starting lba=%" PRId64 " [0x%"
@@ -721,16 +771,18 @@ pt_read(struct opts_t * op, int in0_out1, unsigned char * buff, int blocks,
                 ret = res;
                 goto err_out;
             case -2:
-                pr2serr("ENOMEM again, unexpected (r)\n");
+                pr2serr("%s: ENOMEM again, unexpected [%s]\n", __func__, iop);
                 return -1;
             case SG_LIB_CAT_NOT_READY:
-                pr2serr("device (r) not ready\n");
+                pr2serr("%s: device not ready [%s]\n", __func__, iop);
                 return res;
             case SG_LIB_CAT_UNIT_ATTENTION:
-                pr2serr("Unit attention, unexpected (r)\n");
+                pr2serr("%s: Unit attention, unexpected [%s]\n", __func__,
+                        iop);
                 return res;
             case SG_LIB_CAT_ABORTED_COMMAND:
-                pr2serr("Aborted command, unexpected (r)\n");
+                pr2serr("%s: Aborted command, unexpected [%s]\n", __func__,
+                        iop);
                 return res;
             case SG_LIB_CAT_MEDIUM_HARD_WITH_INFO:
             case SG_LIB_CAT_MEDIUM_HARD:
@@ -742,8 +794,8 @@ pt_read(struct opts_t * op, int in0_out1, unsigned char * buff, int blocks,
                 goto err_out;
             case SG_LIB_SYNTAX_ERROR:
             default:
-                pr2serr(">> unexpected result=%d from pt_low_read() 2\n",
-                        res);
+                pr2serr(">> unexpected result=%d from pt_low_read() 2 [%s]\n",
+                        res, iop);
                 ret = res;
                 goto err_out;
             }
@@ -816,7 +868,7 @@ pt_low_write(struct opts_t * op, const unsigned char * buff, int blocks,
     uint64_t io_addr = 0;
     struct sg_pt_base * ptvp = op->odip->ptvp;
     const struct flags_t * fp = op->oflagp;
-    struct sg_scsi_sense_hdr ssh;
+    const char * desc;
 
     if (pt_build_scsi_cdb(wrCmd, fp->cdbsz, blocks, to_block, 1, fp,
                           op->wrprotect)) {
@@ -824,8 +876,14 @@ pt_low_write(struct opts_t * op, const unsigned char * buff, int blocks,
                 to_block, blocks);
         return SG_LIB_SYNTAX_ERROR;
     }
+    if (DDPT_WRITE_ATOMIC16_OC == wrCmd[0])
+        desc = "WRITE ATOMIC(16)";
+    else if (0xe == (0xf & wrCmd[0]))
+        desc = "WRITE AND VERIFY";
+    else
+        desc = "WRITE";
     if (op->verbose > 2) {
-        pr2serr("    WRITE cdb: ");
+        pr2serr("    %s cdb: ", desc);
         for (k = 0; k < fp->cdbsz; ++k)
             pr2serr("%02x ", wrCmd[k]);
         pr2serr("\n");
@@ -839,13 +897,16 @@ pt_low_write(struct opts_t * op, const unsigned char * buff, int blocks,
     set_scsi_pt_cdb(ptvp, wrCmd, fp->cdbsz);
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     set_scsi_pt_data_out(ptvp, buff, bs * blocks);
+#ifdef SCSI_PT_FLAGS_FUNCTION
+    set_scsi_pt_flags(ptvp, SCSI_PT_FLAGS_QUEUE_AT_TAIL);
+#endif
     vt = (op->verbose ? (op->verbose - 1) : 0);
     while (((res = do_scsi_pt(ptvp, sg_fd, DEF_RW_TIMEOUT, vt)) < 0) &&
            (-EINTR == res))
         ++op->interrupted_retries; /* resubmit if interrupted system call */
 
     vt = ((op->verbose > 1) ? (op->verbose - 1) : op->verbose);
-    ret = sg_cmds_process_resp(ptvp, "WRITE", res, bs * blocks, sense_b,
+    ret = sg_cmds_process_resp(ptvp, desc, res, bs * blocks, sense_b,
                                0 /* noisy */, vt, &sense_cat);
     if (-1 == ret)
         ;
@@ -865,19 +926,15 @@ pt_low_write(struct opts_t * op, const unsigned char * buff, int blocks,
                         PRIx64 ", num=%d\n", to_block, blocks);
             break;
         case SG_LIB_CAT_ABORTED_COMMAND:
-            if (sg_scsi_normalize_sense(sense_b, slen, &ssh) &&
-                (ASC_PROTECTION_INFO_BAD == ssh.asc)) {
-                /* Protection problem, so no retry */
-                ++op->wr_unrecovered_errs;
-                ret = SG_LIB_CAT_PROTECTION;
-            }
-            break;
         case SG_LIB_CAT_UNIT_ATTENTION:
             break;
         case SG_LIB_CAT_NOT_READY:
         case SG_LIB_CAT_ILLEGAL_REQ:
         case SG_LIB_CAT_INVALID_OP:
         case SG_LIB_CAT_SENSE:
+        case SG_LIB_CAT_RES_CONFLICT:
+        case SG_LIB_CAT_DATA_PROTECT:
+        case SG_LIB_CAT_PROTECTION:
             ++op->wr_unrecovered_errs;
             break;
         case SG_LIB_CAT_MEDIUM_HARD:
@@ -892,7 +949,6 @@ pt_low_write(struct opts_t * op, const unsigned char * buff, int blocks,
         }
     } else
         ret = 0;
-
     return ret;
 }
 
@@ -1008,13 +1064,6 @@ pt_write_same16(struct opts_t * op, const unsigned char * buff, int bs,
         ;
     else if (-2 == ret) {
         switch (sense_cat) {
-        case SG_LIB_CAT_NOT_READY:
-        case SG_LIB_CAT_UNIT_ATTENTION:
-        case SG_LIB_CAT_INVALID_OP:
-        case SG_LIB_CAT_ILLEGAL_REQ:
-        case SG_LIB_CAT_ABORTED_COMMAND:
-            ret = sense_cat;
-            break;
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
             ret = 0;
@@ -1033,7 +1082,7 @@ pt_write_same16(struct opts_t * op, const unsigned char * buff, int bs,
             ret = sense_cat;
             break;
         default:
-            ret = -1;
+            ret = sense_cat;
             break;
         }
     } else
@@ -1069,12 +1118,6 @@ pt_tpc_process_res(int cp_ret, int sense_cat, const unsigned char * sense_b,
 
         sb_ok = sg_scsi_normalize_sense(sense_b, sense_len, &ssh);
         switch (sense_cat) {
-        case SG_LIB_CAT_NOT_READY:
-        case SG_LIB_CAT_INVALID_OP:
-        case SG_LIB_CAT_UNIT_ATTENTION:
-        case SG_LIB_CAT_ABORTED_COMMAND:
-            ret = sense_cat;
-            break;
         case SG_LIB_CAT_ILLEGAL_REQ:
             if (sb_ok) {
                 if ((ASC_GENERAL_0 == ssh.asc) &&
@@ -1110,23 +1153,12 @@ pt_tpc_process_res(int cp_ret, int sense_cat, const unsigned char * sense_b,
             } else
                 ret = sense_cat;
             break;
-        case SG_LIB_CAT_SENSE:
-            if (sb_ok) {
-                if (SPC_SK_DATA_PROTECT == ssh.sense_key)
-                    ret = DDPT_CAT_SK_DATA_PROTECT;
-                else if (SPC_SK_COPY_ABORTED == ssh.sense_key)
-                    ret = DDPT_CAT_SK_COPY_ABORTED;
-                else
-                    ret = sense_cat;
-            } else
-                ret = -1;
-            break;
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
             ret = 0;
             break;
         default:
-            ret = -1;
+            ret = sense_cat;
             break;
         }
     } else
@@ -1232,7 +1264,7 @@ pt_3party_copy_out(int sg_fd, int sa, uint32_t list_id, int group_num,
     if ((-1 == ret) &&
         (SCSI_PT_RESULT_STATUS == get_scsi_pt_result_category(ptvp)) &&
         (SAM_STAT_RESERVATION_CONFLICT == get_scsi_pt_status_response(ptvp)))
-        ret = DDPT_CAT_RESERVATION_CONFLICT;
+        ret = SG_LIB_CAT_RES_CONFLICT;
     else
         ret = pt_tpc_process_res(ret, sense_cat, sense_b,
                                  get_scsi_pt_sense_len(ptvp));
@@ -1292,7 +1324,7 @@ pt_3party_copy_in(int sg_fd, int sa, uint32_t list_id, int timeout_secs,
     if ((-1 == ret) &&
         (SCSI_PT_RESULT_STATUS == get_scsi_pt_result_category(ptvp)) &&
         (SAM_STAT_RESERVATION_CONFLICT == get_scsi_pt_status_response(ptvp)))
-        ret = DDPT_CAT_RESERVATION_CONFLICT;
+        ret = SG_LIB_CAT_RES_CONFLICT;
     else
         ret = pt_tpc_process_res(ret, sense_cat, sense_b,
                                  get_scsi_pt_sense_len(ptvp));
