@@ -107,7 +107,7 @@ win32_sleep_ms(int millisecs)
  * placing string in b not exceeding blen bytes. Returns
  * bytes placed in b (excluding trailing NULL) or -1 for
  * error. MS refers to them as "System Error Codes". */
-static int
+static const char *
 win32_errmsg(int errnum, char * b, int blen)
 {
     LPTSTR err_txt = 0;
@@ -115,15 +115,16 @@ win32_errmsg(int errnum, char * b, int blen)
     int len = 0;
 
     if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
-                          FORMAT_MESSAGE_FROM_SYSTEM,
+                      FORMAT_MESSAGE_FROM_SYSTEM,
                       NULL,
                       errn,
                       0,
                       (LPTSTR)&err_txt,
                       0,
-                      NULL) == 0)
-        return -1;
-    else {
+                      NULL) == 0) {
+        snprintf(b, blen, "FormatMessage(errnum=%d) failed", errnum);
+        return b;
+    } else {
         len = strlen(err_txt);
         if (len) {
             if ('\n' == err_txt[len - 1]) {
@@ -145,7 +146,9 @@ win32_errmsg(int errnum, char * b, int blen)
     }
     if (err_txt)
         LocalFree(err_txt);
-    return len;
+    len = strlen(b);
+    snprintf(b + len, blen - len, " [%d]", errnum);
+    return b;
 }
 
 /* Return 1 for filenames starting with '\', or of the form '<letter>:'
@@ -350,7 +353,9 @@ win32_open_if(struct opts_t * op, int flags, int verbose)
     DISK_GEOMETRY g;
     DWORD count, share_mode, err;
     char b[80];
+    int blen;
 
+    blen = sizeof(b);
     if (verbose)
         pr2serr("CreateFile(%s , in)\n", op->idip->fn);
     share_mode = (O_EXCL & flags) ? 0 : (FILE_SHARE_READ | FILE_SHARE_WRITE);
@@ -363,22 +368,14 @@ win32_open_if(struct opts_t * op, int flags, int verbose)
                               NULL);
     if (INVALID_HANDLE_VALUE == op->idip->fh) {
         err = GetLastError();
-        if (win32_errmsg(err, b, sizeof(b)) < 0)
-            pr2serr("CreateFile(in) failed, error=%ld [and win32_errmsg()] "
-                    "failed\n", err);
-        else
-            pr2serr("CreateFile(in) failed, %s [%ld]\n", b, err);
+        pr2serr("CreateFile(in) failed, %s\n", win32_errmsg(err, b, blen));
         return 1;
     }
     if (0 == DeviceIoControl(op->idip->fh, IOCTL_DISK_GET_DRIVE_GEOMETRY,
                              NULL, 0, &g, sizeof(g), &count, NULL)) {
         err = GetLastError();
-        if (win32_errmsg(err, b, sizeof(b)) < 0)
-            pr2serr("DeviceIoControl(in, geometry) failed, error=%ld [and "
-                    "win32_errmsg()] failed\n", err);
-        else
-            pr2serr("DeviceIoControl(in, geometry) failed, %s [%ld]\n", b,
-                    err);
+        pr2serr("DeviceIoControl(in, geometry) failed, %s\n",
+                win32_errmsg(err, b, blen));
         return 1;
     }
     if ((int)g.BytesPerSector != op->ibs) {
@@ -396,7 +393,9 @@ win32_open_of(struct opts_t * op, int flags, int verbose)
     DISK_GEOMETRY g;
     DWORD count, share_mode, err;
     char b[80];
+    int blen;
 
+    blen = sizeof(b);
     if (verbose)
         pr2serr("CreateFile(%s , out)\n", op->odip->fn);
     share_mode = (O_EXCL & flags) ? 0 : (FILE_SHARE_READ | FILE_SHARE_WRITE);
@@ -409,16 +408,14 @@ win32_open_of(struct opts_t * op, int flags, int verbose)
                               NULL);
     if (INVALID_HANDLE_VALUE == op->odip->fh) {
         err = GetLastError();
-        if (win32_errmsg(err, b, sizeof(b)) < 0)
-            pr2serr("CreateFile(out) failed, error=%ld [and win32_errmsg() "
-                    "failed]\n", err);
-        else
-            pr2serr("CreateFile(out) failed, %s [%ld]\n", b, err);
+        pr2serr("CreateFile(out) failed, %s\n", win32_errmsg(err, b, blen));
         return 1;
     }
     if (0 == DeviceIoControl(op->odip->fh, IOCTL_DISK_GET_DRIVE_GEOMETRY,
                              NULL, 0, &g, sizeof(g), &count, NULL)) {
-        pr2serr("DeviceIoControl(out, geometry) error=%ld\n", GetLastError());
+        err = GetLastError();
+        pr2serr("DeviceIoControl(out, geometry) failed, %s\n",
+                win32_errmsg(err, b, blen));
         return 1;
     }
     if ((int)g.BytesPerSector != op->obs) {
@@ -441,7 +438,10 @@ win32_set_file_pos(struct opts_t * op, int which_arg, int64_t pos,
     DWORD lo_ret;
     HANDLE fh;
     const char * cp;
+    char b[80];
+    int blen;
 
+    blen = sizeof(b);
     fh = (DDPT_ARG_IN == which_arg) ? op->idip->fh : op->odip->fh;
     cp = (DDPT_ARG_IN == which_arg) ? "in" : "out";
     if (verbose > 2)
@@ -450,8 +450,8 @@ win32_set_file_pos(struct opts_t * op, int which_arg, int64_t pos,
     if ((INVALID_SET_FILE_POINTER == lo_ret) &&
         (NO_ERROR != (err = GetLastError()))) {
         if (verbose)
-            pr2serr("SetFilePointer failed to set pos=[0x%" PRIx64 "], "
-                    "error=%ld\n", pos, err);
+            pr2serr("SetFilePointer failed to set pos=[0x%" PRIx64 "], %s\n",
+                    pos, win32_errmsg(err, b, blen));
         return 1;
     }
     return 0;
@@ -465,18 +465,15 @@ win32_block_read(struct opts_t * op, unsigned char * bp, int num_bytes,
     DWORD num = num_bytes;
     DWORD howMany, err;
     char b[80];
+    int blen;
 
+    blen = sizeof(b);
     if (verbose > 2)
         pr2serr("ReadFile(num=%d, in)\n", num_bytes);
     if (ReadFile(op->idip->fh, bp, num, &howMany, NULL) == 0) {
         err = GetLastError();
-        if (verbose) {
-            if (win32_errmsg(err, b, sizeof(b)) < 0)
-                pr2serr("ReadFile failed, error=%ld [and win32_errmsg() "
-                        "failed]\n", err);
-            else
-                pr2serr("ReadFile failed, %s [%ld]\n", b, err);
-        }
+        if (verbose)
+            pr2serr("ReadFile failed, %s\n", win32_errmsg(err, b, blen));
         if (23 == err)
             return -SG_LIB_CAT_MEDIUM_HARD;
         else
@@ -493,18 +490,16 @@ win32_block_read_from_of(struct opts_t * op, unsigned char * bp,
     DWORD num = num_bytes;
     DWORD howMany, err;
     char b[80];
+    int blen;
 
+    blen = sizeof(b);
     if (verbose > 2)
         pr2serr("ReadFile(num=%d, out)\n", num_bytes);
     if (ReadFile(op->odip->fh, bp, num, &howMany, NULL) == 0) {
         err = GetLastError();
-        if (verbose) {
-            if (win32_errmsg(err, b, sizeof(b)) < 0)
-                pr2serr("ReadFile failed, error=%ld [and win32_errmsg() "
-                        "failed]\n", err);
-            else
-                pr2serr("ReadFile failed, %s [%ld]\n", b, err);
-        }
+        if (verbose)
+            pr2serr("ReadFile(from_of) failed, %s\n",
+                    win32_errmsg(err, b, blen));
         if (23 == err)
             return -SG_LIB_CAT_MEDIUM_HARD;
         else
@@ -521,18 +516,15 @@ win32_block_write(struct opts_t * op, const unsigned char * bp,
     DWORD num = num_bytes;
     DWORD howMany, err;
     char b[80];
+    int blen;
 
+    blen = sizeof(b);
     if (verbose > 2)
         pr2serr("WriteFile(num=%d, out)\n", num_bytes);
     if (WriteFile(op->odip->fh, bp, num, &howMany, NULL) == 0) {
         err = GetLastError();
-        if (verbose) {
-            if (win32_errmsg(err, b, sizeof(b)) < 0)
-                pr2serr("WriteFile failed, error=%ld [and win32_errmsg() "
-                        "failed]\n", err);
-            else
-                pr2serr("WriteFile failed, %s [%ld]\n", b, err);
-        }
+        if (verbose)
+            pr2serr("WriteFile failed, %s\n", win32_errmsg(err, b, blen));
         if (23 == err)
             return -SG_LIB_CAT_MEDIUM_HARD;
         else
@@ -552,22 +544,27 @@ win32_get_blkdev_capacity(struct opts_t * op, int which_arg,
     DISK_GEOMETRY g;
     GET_LENGTH_INFORMATION gli;
     ULARGE_INTEGER total_bytes;
-    DWORD count;
+    DWORD count, err;
     HANDLE fh;
     const char * fname;
     int64_t byte_len, blks;
     int fname_len;
     char dirName[64];
+    char b[80];
+    int blen;
 
+    blen = sizeof(b);
     fh = (DDPT_ARG_IN == which_arg) ? op->idip->fh : op->odip->fh;
     fname = (DDPT_ARG_IN == which_arg) ? op->idip->fn : op->odip->fn;
     if (op->verbose > 2)
         pr2serr("win32_get_blkdev_capacity: for %s\n", fname);
     if (0 == DeviceIoControl(fh, IOCTL_DISK_GET_DRIVE_GEOMETRY, NULL, 0, &g,
                              sizeof(g), &count, NULL)) {
-        if (op->verbose)
-            pr2serr("DeviceIoControl(blkdev, geometry) error=%ld\n",
-                    GetLastError());
+        if (op->verbose) {
+            err = GetLastError();
+            pr2serr("DeviceIoControl(blkdev, geometry) failed, %s\n",
+                    win32_errmsg(err, b, blen));
+        }
         *num_sect = 0;
         *sect_sz = 0;
         return -1;
@@ -580,9 +577,11 @@ win32_get_blkdev_capacity(struct opts_t * op, int which_arg,
         byte_len = gli.Length.QuadPart;
         *num_sect = byte_len / (int)g.BytesPerSector;
         return 0;
-    } else if (op->verbose > 2)
-        pr2serr("DeviceIoControl(blkdev, length_info) error=%ld\n",
-                GetLastError());
+    } else if (op->verbose > 2) {
+        err = GetLastError();
+        pr2serr("DeviceIoControl(blkdev, length_info) failed, %s\n",
+                win32_errmsg(err, b, blen));
+    }
 
     /* Assume if device name finishes in digit then its physical */
     fname_len = (int)strlen(fname);
@@ -606,9 +605,12 @@ win32_get_blkdev_capacity(struct opts_t * op, int which_arg,
     if (GetDiskFreeSpaceEx(dirName, NULL, &total_bytes, NULL)) {
         byte_len = total_bytes.QuadPart;
         *num_sect = byte_len / (int)g.BytesPerSector;
-    } else if (op->verbose > 1) {
-            pr2serr("GetDiskFreeSpaceEx(%s) error=%ld\n", dirName,
-                    GetLastError());
+    } else {
+        if (op->verbose > 1) {
+            err = GetLastError();
+            pr2serr("GetDiskFreeSpaceEx(%s) failed, %s\n", dirName,
+                    win32_errmsg(err, b, blen));
+        }
         *num_sect = 0;
         return -1;
     }
