@@ -45,7 +45,12 @@
 
 #include "sg_lib.h"
 
-#ifndef _WIN32_WINNT
+#ifdef _WIN32_WINNT
+ #if _WIN32_WINNT < 0x0602
+ #undef _WIN32_WINNT
+ #define _WIN32_WINNT 0x0602
+ #endif
+#else
 #define _WIN32_WINNT 0x0602
 /* claim its W8 */
 #endif
@@ -165,7 +170,7 @@ union STORAGE_DEVICE_DESCRIPTOR_DATA {
 
 union STORAGE_DEVICE_UID_DATA {
     STORAGE_DEVICE_UNIQUE_IDENTIFIER desc;
-    char raw[512];
+    char raw[1060];
 };
 
 struct storage_elem {
@@ -275,20 +280,21 @@ query_dev_property(HANDLE hdevice,
         if (verbose > 2) {
             err = GetLastError();
             pr2serr("  IOCTL_STORAGE_QUERY_PROPERTY(Devprop) failed, "
-                    "Error=%ld %s\n", err, get_err_str(err, sizeof(b), b));
+                    "Error=%u %s\n", (unsigned int)err,
+                    get_err_str(err, sizeof(b), b));
         }
         return -ENOSYS;
     }
 
     if (verbose > 3)
-        pr2serr("  IOCTL_STORAGE_QUERY_PROPERTY(DevProp) num_out=%ld\n",
-                num_out);
+        pr2serr("  IOCTL_STORAGE_QUERY_PROPERTY(DevProp) num_out=%u\n",
+                (unsigned int)num_out);
     return 0;
 }
 
 static int
 query_dev_uid(HANDLE hdevice,
-                    union STORAGE_DEVICE_UID_DATA * data)
+              union STORAGE_DEVICE_UID_DATA * data)
 {
     DWORD num_out, err;
     char b[256];
@@ -296,22 +302,42 @@ query_dev_uid(HANDLE hdevice,
                                     PropertyStandardQuery, {0} };
 
     memset(data, 0, sizeof(*data));
+    num_out = 0;
+    query.QueryType = PropertyExistsQuery;
+    if (! DeviceIoControl(hdevice, IOCTL_STORAGE_QUERY_PROPERTY,
+                          &query, sizeof(query), NULL, 0, &num_out, NULL)) {
+        if (verbose > 2) {
+            err = GetLastError();
+            fprintf(stderr, "  IOCTL_STORAGE_QUERY_PROPERTY(DevUid(exists)) "
+                    "failed, Error=%u %s\n", (unsigned int)err,
+                    get_err_str(err, sizeof(b), b));
+        }
+        if (verbose > 3)
+            fprintf(stderr, "      num_out=%u\n", (unsigned int)num_out);
+        /* interpret any error to mean this property doesn't exist */
+        return 0;
+    }
+
+    query.QueryType = PropertyStandardQuery;
     if (! DeviceIoControl(hdevice, IOCTL_STORAGE_QUERY_PROPERTY,
                           &query, sizeof(query), data, sizeof(*data),
                           &num_out, NULL)) {
         if (verbose > 2) {
             err = GetLastError();
-            pr2serr("  IOCTL_STORAGE_QUERY_PROPERTY(DevUid) failed, "
-                    "Error=%ld %s\n", err, get_err_str(err, sizeof(b), b));
+            fprintf(stderr, "  IOCTL_STORAGE_QUERY_PROPERTY(DevUid) failed, "
+                    "Error=%u %s\n", (unsigned int)err,
+                    get_err_str(err, sizeof(b), b));
         }
         return -ENOSYS;
     }
     if (verbose > 3)
-        pr2serr("  IOCTL_STORAGE_QUERY_PROPERTY(DevUid) num_out=%ld\n",
-                num_out);
+        fprintf(stderr, "  IOCTL_STORAGE_QUERY_PROPERTY(DevUid) num_out=%u\n",
+                (unsigned int)num_out);
     return 0;
 }
 
+/* Updates storage_arr based on sep. Returns 1 if update occurred, 0 if
+ * no update occured. */
 static int
 check_devices(const struct storage_elem * sep)
 {
@@ -321,7 +347,18 @@ check_devices(const struct storage_elem * sep)
     for (k = 0; k < next_unused_elem; ++k, ++sarr) {
         if ('\0' == sarr->name[0])
             continue;
-        if (sep->qp_descriptor_valid && sarr->qp_descriptor_valid) {
+        if (sep->qp_uid_valid && sarr->qp_uid_valid) {
+            if (0 == memcmp(&sep->qp_uid, &sarr->qp_uid,
+                            sizeof(sep->qp_uid))) {
+                for (j = 0; j < (int)sizeof(sep->volume_letters); ++j) {
+                    if ('\0' == sarr->volume_letters[j]) {
+                        sarr->volume_letters[j] = sep->name[0];
+                        break;
+                    }
+                }
+                return 1;
+            }
+        } else if (sep->qp_descriptor_valid && sarr->qp_descriptor_valid) {
             if (0 == memcmp(&sep->qp_descriptor, &sarr->qp_descriptor,
                             sizeof(sep->qp_descriptor))) {
                 for (j = 0; j < (int)sizeof(sep->volume_letters); ++j) {
@@ -333,7 +370,6 @@ check_devices(const struct storage_elem * sep)
                 return 1;
             }
         }
-        // should do uid check here (probably before descriptor compare)
     }
     return 0;
 }
@@ -349,7 +385,7 @@ enum_scsi_adapters(void)
     BYTE bus;
     BOOL success;
     char adapter_name[64];
-    char inqDataBuff[2048];
+    char inqDataBuff[8192];
     PSCSI_ADAPTER_BUS_INFO  ai;
     char b[256];
 
@@ -362,7 +398,7 @@ enum_scsi_adapters(void)
             hole_count = 0;
             success = DeviceIoControl(fh, IOCTL_SCSI_GET_INQUIRY_DATA,
                                       NULL, 0, inqDataBuff,
-                                      sizeof(inqDataBuff), &dummy, FALSE);
+                                      sizeof(inqDataBuff), &dummy, NULL);
             if (success) {
                 PSCSI_BUS_DATA pbd;
                 PSCSI_INQUIRY_DATA pid;
@@ -395,17 +431,19 @@ enum_scsi_adapters(void)
                 }
             } else {
                 err = GetLastError();
-                pr2serr("%s: IOCTL_SCSI_GET_INQUIRY_DATA failed err=%lu\n"
-                        "\t%s", adapter_name, err,
+                pr2serr("%s: IOCTL_SCSI_GET_INQUIRY_DATA failed err=%u\n"
+                        "\t%s", adapter_name, (unsigned int)err,
                         get_err_str(err, sizeof(b), b));
             }
             CloseHandle(fh);
         } else {
-            if (verbose > 3) {
-                err = GetLastError();
-                pr2serr("%s: CreateFile failed err=%lu\n\t%s", adapter_name,
-                        err, get_err_str(err, sizeof(b), b));
-            }
+            err = GetLastError();
+            if (ERROR_SHARING_VIOLATION == err)
+                fprintf(stderr, "%s: in use by other process (sharing "
+                        "violation [34])\n", adapter_name);
+            else if (verbose > 3)
+                pr2serr("%s: CreateFile failed err=%u\n\t%s", adapter_name,
+                        (unsigned int)err, get_err_str(err, sizeof(b), b));
             if (++hole_count >= MAX_HOLE_COUNT)
                 break;
         }
@@ -483,11 +521,13 @@ enum_pds(void)
             memcpy(&storage_arr[next_unused_elem++], &tmp_se, sizeof(tmp_se));
             CloseHandle(fh);
         } else {
-            if (verbose > 3) {
-                err = GetLastError();
-                pr2serr("%s: CreateFile failed err=%lu\n\t%s", adapter_name,
-                        err, get_err_str(err, sizeof(b), b));
-            }
+            err = GetLastError();
+            if (ERROR_SHARING_VIOLATION == err)
+                fprintf(stderr, "%s: in use by other process (sharing "
+                        "violation [34])\n", adapter_name);
+            else if (verbose > 3)
+                pr2serr("%s: CreateFile failed err=%u\n\t%s", adapter_name,
+                        (unsigned int)err, get_err_str(err, sizeof(b), b));
             if (++hole_count >= MAX_HOLE_COUNT)
                 break;
         }
@@ -529,11 +569,13 @@ enum_cdroms(void)
             memcpy(&storage_arr[next_unused_elem++], &tmp_se, sizeof(tmp_se));
             CloseHandle(fh);
         } else {
-            if (verbose > 3) {
-                err = GetLastError();
-                pr2serr("%s: CreateFile failed err=%lu\n\t%s", adapter_name,
-                        err, get_err_str(err, sizeof(b), b));
-            }
+            err = GetLastError();
+            if (ERROR_SHARING_VIOLATION == err)
+                fprintf(stderr, "%s: in use by other process (sharing "
+                        "violation [34])\n", adapter_name);
+            else if (verbose > 3)
+                pr2serr("%s: CreateFile failed err=%u\n\t%s", adapter_name,
+                        (unsigned int)err, get_err_str(err, sizeof(b), b));
             if (++hole_count >= MAX_HOLE_COUNT)
                 break;
         }
@@ -575,11 +617,13 @@ enum_tapes(void)
             memcpy(&storage_arr[next_unused_elem++], &tmp_se, sizeof(tmp_se));
             CloseHandle(fh);
         } else {
-            if (verbose > 3) {
-                err = GetLastError();
-                pr2serr("%s: CreateFile failed err=%lu\n\t%s", adapter_name,
-                        err, get_err_str(err, sizeof(b), b));
-            }
+            err = GetLastError();
+            if (ERROR_SHARING_VIOLATION == err)
+                fprintf(stderr, "%s: in use by other process (sharing "
+                        "violation [34])\n", adapter_name);
+            else if (verbose > 3)
+                pr2serr("%s: CreateFile failed err=%u\n\t%s", adapter_name,
+                        (unsigned int)err, get_err_str(err, sizeof(b), b));
             if (++hole_count >= MAX_HOLE_COUNT)
                 break;
         }
@@ -643,9 +687,13 @@ do_wscan(char letter, int show_bt, int scsi_scan)
                     printf("%s", sp->qp_descriptor.raw + j);
                 printf("\n");
                 if (verbose > 2)
-                    dStrHex(sp->qp_descriptor.raw, 144, 0);
+                    dStrHexErr(sp->qp_descriptor.raw, 144, 0);
             } else
                 printf("\n");
+            if ((verbose > 3) && sp->qp_uid_valid) {
+                printf("  UID valid, in hex:\n");
+                dStrHexErr(sp->qp_uid.raw, sizeof(sp->qp_uid.raw), 1);
+            }
         }
     }
 
